#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.
#

# Below are the steps to prepare the source files.
#
# Use a specified version as HEAD if needed, e.g. "v4.14.0"
#
# # Source0:
# git checkout <version tag>
# TAG=$(git log --pretty="format:%h" -n 1)
# distdir="ClusterLabs-resource-agents-${TAG}"
# TARFILE="${distdir}.tar.gz"
# rm -rf $TARFILE $distdir
# git archive --prefix=$distdir/ HEAD | gzip > $TARFILE
#
# # Source1:
#
# 1) The script to generate a new source file from the SAPHanaSR
#    upstream git repo.
#
#      git switch main
#      TAG=$(git log --pretty="format:%h" -n 1)
#      distdir="sap-hana-ha-${TAG}"
#      TARFILE="${distdir}.tar.gz"
#      rm -rf $TARFILE $distdir
#      git archive --prefix=$distdir/ HEAD | gzip > $TARFILE
#
# 2) Update sources with pattern replacements.
#    Run ONCE and include the patches. Only re-run if patch conflictcs
#    appeared and were solved.
#
#      sh -x ./create-rename-patches.sh
#
# Always carefully review the results of the patches, there is no
# automatic check elsewhere!

%global upstream_prefix ClusterLabs-resource-agents
%global upstream_version a6ccb93a

# Use %%{name}
# %%global name sap-hana-ha
%global saphana_hash 87a5781

# Whether this platform defaults to using systemd as an init system
# (needs to be evaluated prior to BuildRequires being enumerated and
# installed as it's intended to conditionally select some of these, and
# for that there are only few indicators with varying reliability:
# - presence of systemd-defined macros (when building in a full-fledged
#   environment, which is not the case with ordinary mock-based builds)
# - systemd-aware rpm as manifested with the presence of particular
#   macro (rpm itself will trivially always be present when building)
# - existence of /usr/lib/os-release file, which is something heavily
#   propagated by systemd project
# - when not good enough, there's always a possibility to check
#   particular distro-specific macros (incl. version comparison)
%define systemd_native (%{?_unitdir:1}%{!?_unitdir:0}%{nil \
  } || %{?__transaction_systemd_inhibit:1}%{!?__transaction_systemd_inhibit:0}%{nil \
  } || %(test -f /usr/lib/os-release; test $? -ne 0; echo $?))

# determine the ras-set to process based on configure invocation
%bcond_with rgmanager
%bcond_without linuxha

Name:     sap-hana-ha
Summary:  Cluster resource HA for SAP HANA in System Replication
Version:  1.2.8
Release:  3%{?rcver:%{rcver}}%{?numcomm:.%{numcomm}}%{?alphatag:.%{alphatag}}%{?dirty:.%{dirty}}%{?dist}
License:  GPLv2+
URL:      https://github.com/SUSE/SAPHanaSR
Source0:  %{upstream_prefix}-%{upstream_version}.tar.gz
Source1:  %{name}-%{saphana_hash}.tar.gz
# Unpackaged source, just to make sure centpkg keeps it.
Source2:  create-rename-patches.sh

# Patches that will always be needed.
# Other patches: handle conflicts if diff lines overlap
Patch100: patterns-srHook-files.patch
Patch101: patterns-ra-files.patch
Patch102: patterns-tools-files.patch

BuildArch:  noarch

# Build dependencies
BuildRequires: make
BuildRequires: automake autoconf pkgconfig gcc
BuildRequires: libxslt glib2-devel libqb-devel
BuildRequires: systemd
BuildRequires: which

%if 0%{?fedora} || 0%{?centos} > 7 || 0%{?rhel} > 7 || 0%{?suse_version}
BuildRequires: python3-devel
%else
BuildRequires: python-devel
%endif

%if 0%{?fedora} || 0%{?centos} || 0%{?rhel}
BuildRequires: docbook-style-xsl docbook-dtds
%if 0%{?rhel} == 0
BuildRequires: libnet-devel
%endif
%endif

%if 0%{?suse_version}
BuildRequires:  libnet-devel
%if 0%{?suse_version} > 1500
BuildRequires:  cluster-glue-devel
%else
BuildRequires:  libglue-devel
%endif
BuildRequires:  libxslt docbook_4 docbook-xsl-stylesheets
%endif

Requires: resource-agents >= 4.8.0

# Package dependencies
Requires:   resource-agents >= 4.8.0
Conflicts:  resource-agents-sap-hana
Conflicts:  resource-agents-sap-hana-scaleout

Requires: /bin/bash /usr/bin/grep /bin/sed /bin/gawk
Requires: python3


%description
The SAP HANA resource agents and hook scripts interface with Pacemaker
to allow SAP HANA scale-up and scale-out instances to be managed in a cluster
environment.


%prep
# Unpack the Clusterlabs source
%setup -q -n %{upstream_prefix}-%{upstream_version}
%setup -q -T -D -a 1 -n %{upstream_prefix}-%{upstream_version}

# Rename the unpacked SAP dir to strip the suffix off.
mv %{name}-%{saphana_hash} %{name}

# Apply the patches that fix some names and patterns
%autopatch -p0

# Fail if there are patterns found after patching.
# Recreate the patches if the source files changed.
# Return success message for the final command RC=0.
grep -r \
  -e "\"provider_company\": \"SUSE\"" \
  -e "susChkSrv" \
  -e "suschksrv" \
  -e "susTkOver" \
  -e "sustkover" \
  -e "susHanaSR" \
  -e "sushanasr" \
  -e "SAPHanaSR-angi" \
  -e "suse_SAPHanaFilesystem" \
  %{name}/tools/ %{name}/srHook/ %{name}/ra/ && {
    echo -e "\v>>> CHECK FAILED: Patterns found after patching. Fix the patches.\v"
    exit 1
  } || echo "CHECK PASSED: No unpatched patterns found"


### The following is required to get the files in a shape that the compilation
### and man page generation through the Clusterlabs code works.
### The files are moved temporarily to places, adjusted to work at compilation time,
### but the package will use the files from their original path of the SAP package source.

# Copy the agents and libs to the heartbeat dir for the ClusterLabs included processing.
# Keep them in the original place! They are used for the package.
cp %{name}/ra/SAP* heartbeat
cp %{name}/ra/sap* heartbeat

# Required for working in the buildroot:
# Fix the paths to source the r-a libs in their temporary build path.
sed -i -e 's#/usr/lib/%{name}/#${OCF_FUNCTIONS_DIR}/#g' heartbeat/SAPHana*

# Find the existing SAPInstance entry in the list and add new after in corresponding formatting.
# heartbeat/Makefile.am indents by 3 tabs in the target list.
# Skip the .in suffix, the agent is renamed in the process.
sed -i -e 's/\(\t\tSAPInstance\t\t\\\)/\1\n\t\t\tSAPHanaController\t\t\\/' heartbeat/Makefile.am
sed -i -e 's/\(\t\tSAPInstance\t\t\\\)/\1\n\t\t\tSAPHanaFilesystem\t\t\\/' heartbeat/Makefile.am
sed -i -e 's/\(\t\tSAPInstance\t\t\\\)/\1\n\t\t\tSAPHanaTopology\t\t\\/' heartbeat/Makefile.am
# Add the libs after sapdb.sh in ocfcommon_DATA.
# The block indents by 3 tabs + 2 spaces.
sed -i -e 's/\(\t\t\t  sapdb.sh\t\t\\\)/\1\n\t\t\t  saphana-common-lib\t\\/' heartbeat/Makefile.am
sed -i -e 's/\(\t\t\t  sapdb.sh\t\t\\\)/\1\n\t\t\t  saphana-controller-common-lib\t\\/' heartbeat/Makefile.am
sed -i -e 's/\(\t\t\t  sapdb.sh\t\t\\\)/\1\n\t\t\t  saphana-controller-lib\t\\/' heartbeat/Makefile.am
sed -i -e 's/\(\t\t\t  sapdb.sh\t\t\\\)/\1\n\t\t\t  saphana-filesystem-lib\t\\/' heartbeat/Makefile.am
sed -i -e 's/\(\t\t\t  sapdb.sh\t\t\\\)/\1\n\t\t\t  saphana-topology-lib\t\\/' heartbeat/Makefile.am

# Find the existing SAPInstance entry in the list and add new after in corresponding formatting.
# The block in doc/man/Makefile.am indents by 26 spaces in the target list.
sed -i -e 's/\( \{26\}ocf_heartbeat_SAPInstance.7 \\\)/\1\n'\
'                          ocf_heartbeat_SAPHanaController.7 \\/' doc/man/Makefile.am
sed -i -e 's/\( \{26\}ocf_heartbeat_SAPInstance.7 \\\)/\1\n'\
'                          ocf_heartbeat_SAPHanaFilesystem.7 \\/' doc/man/Makefile.am
sed -i -e 's/\( \{26\}ocf_heartbeat_SAPInstance.7 \\\)/\1\n'\
'                          ocf_heartbeat_SAPHanaTopology.7 \\/' doc/man/Makefile.am

### build and configure sections are needed for the build of the Clusterlabs source
### and the resulting man pages that we need.
%build
if [ ! -f configure ]; then
  ./autogen.sh
fi

%if 0%{?fedora} >= 11 || 0%{?centos} > 5 || 0%{?rhel} > 5
CFLAGS="$(echo '%{optflags}')"
%global conf_opt_fatal "--enable-fatal-warnings=no"
%else
CFLAGS="${CFLAGS} ${RPM_OPT_FLAGS}"
%global conf_opt_fatal "--enable-fatal-warnings=yes"
%endif

%if %{with rgmanager}
%global rasset rgmanager
%endif
%if %{with linuxha}
%global rasset linux-ha
%endif
%if %{with rgmanager} && %{with linuxha}
%global rasset all
%endif

export CFLAGS

%configure \
%if 0%{?fedora} || 0%{?centos} > 7 || 0%{?rhel} > 7 || 0%{?suse_version}
  PYTHON="%{__python3}" \
%endif
  %{conf_opt_fatal} \
%if %{defined _unitdir}
    --with-systemdsystemunitdir=%{_unitdir} \
%endif
%if %{defined _tmpfilesdir}
    --with-systemdtmpfilesdir=%{_tmpfilesdir} \
    --with-rsctmpdir=/run/resource-agents \
%endif
  --with-pkg-name=resource-agents \
  --with-ras-set=%{rasset}

make %{_smp_mflags}

%install
rm -rf %{buildroot}
make install DESTDIR=%{buildroot}

# Remove all ocf contents and unrelated man pages
find %{buildroot}/usr/lib/ocf ! -type d -exec rm {} \;
find %{buildroot}%{_mandir} -type f ! -iname "*SAP*" -exec rm {} \;

# Dir structure
mkdir -p %{buildroot}%{_bindir}
mkdir -p %{buildroot}%{_datadir}/%{name}/samples
mkdir -p %{buildroot}%{_usr}/lib/%{name}
mkdir -p %{buildroot}%{_usr}/lib/ocf/resource.d/heartbeat

# Resource agents and r-a libraries
install -m 0755 %{name}/ra/SAPHana* %{buildroot}%{_usr}/lib/ocf/resource.d/heartbeat
install -m 0644 %{name}/ra/saphana-*-lib %{buildroot}%{_usr}/lib/%{name}

# Hook scripts - sanitize upstream names where needed
install -m 0644 %{name}/srHook/susHanaSR.py %{buildroot}%{_datadir}/%{name}/HanaSR.py
install -m 0644 %{name}/srHook/susChkSrv.py %{buildroot}%{_datadir}/%{name}/ChkSrv.py

# Select sample config files
install -m 0444 %{name}/srHook/global.ini_susHanaSR %{buildroot}/%{_datadir}/%{name}/samples/global.ini_HanaSR
install -m 0444 %{name}/srHook/global.ini_susChkSrv %{buildroot}/%{_datadir}/%{name}/samples/global.ini_ChkSrv

# Scripts for some add-on functionality.
# saphana_sr_tools is a lib imported by SAPHanaSR-* python scripts.
install -m 0555 %{name}/tools/SAPHanaSR-showAttr %{buildroot}%{_bindir}
install -m 0444 %{name}/tools/saphana_sr_tools.py %{buildroot}%{_usr}/lib/%{name}

## tree fixup
# remove docs (there is only one and they should come from doc sections in files)
rm -rf %{buildroot}/usr/share/doc/resource-agents


%files
%defattr(-,root,root)
%license %{name}/LICENSE
%dir %{_datadir}/%{name}
%dir %{_usr}/lib/%{name}
%{_bindir}/*
%{_datadir}/%{name}/*
%{_mandir}/man7/*SAPHana*
%{_usr}/lib/ocf/resource.d/heartbeat/SAPHana*
%{_usr}/lib/%{name}/*

%exclude /etc
%exclude /usr/include
%exclude %{_usr}/lib/debug
%exclude %{_usr}/lib/tmpfiles.d
%exclude /usr/libexec/heartbeat
%exclude /usr/lib/systemd
%exclude /usr/sbin/ldirectord
%exclude /usr/sbin/ocf*
%exclude /usr/share/man/man7/*SAPDatabase*
%exclude /usr/share/man/man7/*SAPInstance*
%exclude /usr/share/resource-agents
%exclude /usr/share/pkgconfig/resource-agents.pc
%exclude /usr/src

%changelog
* Fri Jan 17 2025 Janine Fuchs <jfuchs@redhat.com> - 1.2.8-3
- Include man pages.

  Resolves: RHEL-74289

* Tue Oct 29 2024 Troy Dawson <tdawson@redhat.com> - 1.2.8-2.1
- Bump release for October 2024 mass rebuild:
  Resolves: RHEL-64018

* Tue Aug 13 2024 Janine Fuchs <jfuchs@redhat.com> - 1.2.8-2
- Add package to RHEL 10.

  Resolves: RHEL-53598


# vim:set ai ts=2 sw=2 sts=2 et:
